#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2020 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

import math
import sip

from gnuradio import gr
from gnuradio import qtgui
from gnuradio import blocks, fft, filter

from PyQt5 import QtGui
from PyQt5.QtWidgets import QWidget

class Normalize(gr.hier_block2):
    def __init__(self, vecsize=1024):
        gr.hier_block2.__init__(
            self, "Normalize",
            gr.io_signature(1, 1, gr.sizeof_float*vecsize),
            gr.io_signature(1, 1, gr.sizeof_float*vecsize),
        )

        ##################################################
        # Parameters
        ##################################################
        self.vecsize = vecsize

        ##################################################
        # Blocks
        ##################################################
        self.blocks_stream_to_vector_0 = blocks.stream_to_vector(gr.sizeof_float, vecsize)
        self.blocks_repeat_0 = blocks.repeat(gr.sizeof_float, vecsize)
        self.blocks_max_xx_0 = blocks.max_ff(vecsize)
        self.blocks_divide_xx_0 = blocks.divide_ff(vecsize)

        ##################################################
        # Connections
        ##################################################
        self.connect((self.blocks_divide_xx_0, 0), (self, 0))
        self.connect((self.blocks_stream_to_vector_0, 0), (self.blocks_divide_xx_0, 1))
        self.connect((self, 0), (self.blocks_max_xx_0, 0))
        self.connect((self.blocks_repeat_0, 0), (self.blocks_stream_to_vector_0, 0))
        self.connect((self.blocks_max_xx_0, 0), (self.blocks_repeat_0, 0))
        self.connect((self, 0), (self.blocks_divide_xx_0, 0))


    def get_vecsize(self):
        return self.vecsize

    def set_vecsize(self, vecsize):
        self.vecsize = vecsize

class AutoCorrelator(gr.hier_block2):
    """
    This block uses the Wiener Khinchin theorem that the FFT of a signal's
    power spectrum is its auto-correlation function.
    FAC Size controls the FFT size and therefore the length of time
    (samp_rate/fac_size) the auto-correlation runs over.
    """
    def __init__(self, sample_rate, fac_size, fac_decimation, use_db):
        gr.hier_block2.__init__(self,"AutoCorrelator",
            gr.io_signature(1, 1, gr.sizeof_gr_complex), # Input sig
            gr.io_signature(1, 1, gr.sizeof_float*fac_size)) # Output sig

        self.fac_size = fac_size
        self.fac_decimation = fac_decimation
        self.sample_rate = sample_rate

        streamToVec = blocks.stream_to_vector(gr.sizeof_gr_complex, self.fac_size)
        # Make sure N is at least 1
        decimation =  int(self.sample_rate/self.fac_size/self.fac_decimation)
        self.one_in_n = blocks.keep_one_in_n(gr.sizeof_gr_complex * self.fac_size, max(1, decimation))

        # FFT Note: No windowing.
        fac = fft.fft_vcc(self.fac_size, True, ())

        complex2Mag = blocks.complex_to_mag(self.fac_size)
        self.avg = filter.single_pole_iir_filter_ff(1.0, self.fac_size)

        fac_fac   = fft.fft_vfc(self.fac_size, True, ())
        fac_c2mag = blocks.complex_to_mag(fac_size)

        # There's a note in Baz's block about needing to add 3 dB to each bin but the DC bin, however it was never implemented
        n = 20
        k = -20 * math.log10(self.fac_size)
        log = blocks.nlog10_ff(n, self.fac_size, k)

        if use_db:
            self.connect(self, streamToVec, self.one_in_n, fac, complex2Mag, fac_fac, fac_c2mag, self.avg, log, self)
        else:
            self.connect(self, streamToVec, self.one_in_n, fac, complex2Mag, fac_fac, fac_c2mag, self.avg, self)

class AutoCorrelatorSink(gr.hier_block2):
    """
    docstring for block AutoCorrelatorSink
    """
    def __init__(self, sample_rate, fac_size, fac_decimation, title, autoScale, grid, yMin, yMax, use_db):
        gr.hier_block2.__init__(self,
            "AutoCorrelatorSink",
            gr.io_signature(1, 1, gr.sizeof_gr_complex), # Input signature
            gr.io_signature(0, 0, 0)) # Output signature

        self.fac_size = fac_size
        self.fac_decimation = fac_decimation
        self.sample_rate = sample_rate

        autoCorr = AutoCorrelator(sample_rate, fac_size, fac_decimation, use_db)
        vecToStream = blocks.vector_to_stream(gr.sizeof_float, self.fac_size)

        self.timeSink = qtgui.time_sink_f(self.fac_size//2, sample_rate, title, 1, None)
        self.timeSink.enable_grid(grid)
        self.timeSink.set_y_axis(yMin, yMax)
        self.timeSink.enable_autoscale(autoScale)
        self.timeSink.disable_legend()
        self.timeSink.set_update_time(0.1)

        if use_db:
            self.connect(self, autoCorr, vecToStream, self.timeSink)
        else:
            norm = Normalize(self.fac_size)
            self.connect(self, autoCorr, norm, vecToStream, self.timeSink)


    def getWidget(self):
        return sip.wrapinstance(self.timeSink.pyqwidget(), QWidget)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2020 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

from PyQt5 import QtWidgets
import numpy as np
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure
import math

from gnuradio import gr
import pmt

class AzElPlot(gr.sync_block, FigureCanvas):
    """
    This block creates a polar plot with azimuth represented as the angle
    clockwise around the circle, and elevation represented as the radius.
    90 degrees elevation is center (directly overhead),
    while the horizon (0 degrees) is the outside circe.  Note that if an
    elevation < 0 is provided, the marker will turn to an open circle
    on the perimeter at the specified azimuth angle.
    """
    def __init__(self, lbl, backgroundColor, dotColor, Parent=None,
                 width=4, height=4, dpi=90):
        gr.sync_block.__init__(self, name = "MsgPushButton", in_sig = None,
                               out_sig = None)

        self.lbl = lbl

        self.message_port_register_in(pmt.intern("azel"))
        self.set_msg_handler(pmt.intern("azel"), self.msgHandler)

        self.dotColor = dotColor
        self.backgroundColor = backgroundColor
        self.scaleColor = 'black'
        if (self.backgroundColor == 'black'):
            self.scaleColor = 'white'

        self.fig = Figure(figsize=(width, height), dpi=dpi)
        self.fig.patch.set_facecolor(self.backgroundColor)

        self.axes = self.fig.add_subplot(111, polar=True, facecolor=self.backgroundColor)

        # Create an "invisible" line at 90 to set the max for the plot
        self.axes.plot(np.linspace(0, 2*np.pi, 90), np.ones(90)*90, color=self.scaleColor,
                       linestyle='')

        # Plot line: Initialize out to 90 and blank
        radius = 90
        self.blackline = self.axes.plot(np.linspace(0, 2*np.pi, 90), np.ones(90)*radius,
                                        color=self.scaleColor, linestyle='-')
        self.reddot = None

        # Rotate zero up
        self.axes.set_theta_zero_location("N")

        # Set limits:
        self.axes.set_rlim(0, 90)

        self.axes.set_yticklabels([], color=self.scaleColor)
        self.axes.set_xticklabels(['0', '315', '270', '225', '180', '135', '90', '45'],
                                  color=self.scaleColor)

        FigureCanvas.__init__(self, self.fig)
        self.setParent(Parent)

        self.title = self.fig.suptitle(self.lbl, fontsize=8, fontweight='bold', color='black')

        FigureCanvas.setSizePolicy(self,
                                   QtWidgets.QSizePolicy.Expanding,
                                   QtWidgets.QSizePolicy.Expanding)

        self.setMinimumSize(240, 230)
        FigureCanvas.updateGeometry(self)

    def msgHandler(self, msg):
        new_val = None

        try:
            new_val = pmt.to_python(pmt.car(msg))
            if new_val is not None:
                if type(new_val) == dict:
                    if 'az' in new_val and 'el' in new_val:
                        self.updateData(float(new_val['az']), float(new_val['el']))
                    else:
                        gr.log.error("az and el keys were not found in the dictionary.")
                else:
                    gr.log.error("Value received was not a dictionary.  Expecting a dictionary "
                                 "in the car message component with az and el keys.")
            else:
                gr.log.error("The CAR section of the inbound message was None.  "
                      "This part should contain the dictionary with 'az' and 'el' float keys.")
        except Exception as e:
            gr.log.error("[AzElPlot] Error with message conversion: %s" % str(e))
            if new_val is not None:
                gr.log.error(str(new_val))

    def updateData(self, azimuth, elevation):
        if self.reddot is not None:
            self.reddot.pop(0).remove()

        # Plot is angle, radius where angle is in radians

        if (elevation > 0):
            # Need to reverse elevation.  90 degrees is center (directly overhead),
            # and 90 degrees is horizon.
            if (elevation > 90.0):
                elevation = 90.0

            convertedElevation = 90.0 - elevation
            # Note: +azimuth for the plot is measured counter-clockwise, so need to reverse it.
            self.reddot = self.axes.plot(-azimuth * math.pi/180.0, convertedElevation, self.dotColor,
                                         markersize=8)
        else:
            # It's below the horizon.  Show an open circle at the perimeter
            elevation = 0.0
            self.reddot = self.axes.plot(-azimuth * math.pi/180.0, 89.0, self.dotColor,
                                         markerfacecolor="None", markersize=16, fillstyle=None)

        self.draw()

